@@ -1,7 +1,10 @@
 import jwt
+
 from rest_framework import exceptions
+from rest_framework.compat import smart_text
 from rest_framework.authentication import (BaseAuthentication,
                                            get_authorization_header)
+
 from rest_framework_jwt.settings import api_settings
 
 try:
@@ -13,6 +16,7 @@ else:
 
 
 jwt_decode_handler = api_settings.JWT_DECODE_HANDLER
+jwt_get_user_id_from_payload = api_settings.JWT_PAYLOAD_GET_USER_ID_HANDLER
 
 
 class JSONWebTokenAuthentication(BaseAuthentication):
@@ -20,7 +24,8 @@ class JSONWebTokenAuthentication(BaseAuthentication):
     Token based authentication using the JSON Web Token standard.
 
     Clients should authenticate by passing the token key in the "Authorization"
-    HTTP header, prepended with the string "JWT ".  For example:
+    HTTP header, prepended with the string specified in the setting
+    `JWT_AUTH_HEADER_PREFIX`. For example:
 
         Authorization: JWT eyJhbGciOiAiSFMyNTYiLCAidHlwIj
     """
@@ -32,15 +37,16 @@ class JSONWebTokenAuthentication(BaseAuthentication):
         supplied using JWT-based authentication.  Otherwise returns `None`.
         """
         auth = get_authorization_header(request).split()
+        auth_header_prefix = api_settings.JWT_AUTH_HEADER_PREFIX.lower()
 
-        if not auth or auth[0].lower() != b'jwt':
+        if not auth or smart_text(auth[0].lower()) != auth_header_prefix:
             return None
 
         if len(auth) == 1:
-            msg = 'Invalid JWT header. No credentials provided.'
+            msg = 'Invalid Authorization header. No credentials provided.'
             raise exceptions.AuthenticationFailed(msg)
         elif len(auth) > 2:
-            msg = ('Invalid JWT header. Credentials string '
+            msg = ('Invalid Authorization header. Credentials string '
                    'should not contain spaces.')
             raise exceptions.AuthenticationFailed(msg)
 
@@ -62,7 +68,7 @@ class JSONWebTokenAuthentication(BaseAuthentication):
         Returns an active user that matches the payload's user id and email.
         """
         try:
-            user_id = payload.get('user_id')
+            user_id = jwt_get_user_id_from_payload(payload)
 
             if user_id:
                 user = User.objects.get(pk=user_id, is_active=True)
